#HeroCloud.py

import math

import pygame

from pygame import Rect

from engine.AtlanSprite import AtlanSprite

from common.Utility import *

import random



class HeroCloud(AtlanSprite):
	"""
		The protagonist for A Cloudy Day.
	"""

	def __init__(self, myBounds):
		"""
			Generate a bunch of cloud specs around a certain x,y coordinate.
			Unlike normal clouds, the circles are kept in a list so their color can be changed.
		"""
		AtlanSprite.__init__(self)
		self.bounds = myBounds
		self.flowCounter = 0
		self.windStrength = 0
		self.windDirection = 0
		self.mySpeed = (1.0 / (self.bounds.width * self.bounds.height)) * 2000
		self.center = (self.bounds.x + (self.bounds.width / 2), self.bounds.y + (self.bounds.height / 2))
		self.lowestX = 9000
		self.highestX = -9000
		self.highestY = -9000
		#all the circles that make up this procedural cloud
		self.circles = []
		for i in xrange((self.bounds.width * self.bounds.height) / 100):
			#center = placed randomly within bounds (30% to 70% section)
			centerX = random.randint(int(self.bounds.width * 0.3), int(self.bounds.width * 0.7))
			centerY = random.randint(int(self.bounds.height * 0.3), int(self.bounds.height * 0.7))
			radius = random.randint(3, 10)
			self.circles.append([centerX, centerY, radius])
			if centerX - radius < self.lowestX:
                                self.lowestX = centerX - radius
                        if centerX + radius > self.highestX:
                                self.highestX = centerX + radius
                        if centerY + radius > self.highestY:
                                self.highestY = centerY + radius
                self.bounds.width = self.highestX - self.lowestX
		self.trueX = self.bounds.x / 1.0
		self.trueY = self.bounds.y / 1.0
		self.energy = 100.0
		self.maxEnergy = 100.0
		self.energyCounter = 50
		
	def draw(self, surface):
		"""
			Draw the cloud to the surface.
			Color depends on current level of energy
		"""
		for circle in self.circles:
                        color = int((self.energy / self.maxEnergy) * 128)
                        color = 255 - color
                        pygame.draw.circle(surface, (color, color, color, 128), (circle[0] + self.bounds.x, circle[1] + self.bounds.y), circle[2])
                        #surface.blit(self.proSurface, self.bounds)
		
	def update(self, delta):
		"""
			Called every frame to update logic.
			Position changes based on wind speed.
			Slowly regains lost energy over time.
		"""
		self.flowCounter += delta
		if self.flowCounter > 0:
			self.flowCounter = 0
			self.flow(self.windStrength, self.windDirection)
		self.energyCounter -= delta
		if self.energyCounter <= 0:
                        self.energyCounter = 50
                        self.energy = min(self.energy + 0.7, self.maxEnergy)
			
		#If in the start screen, the clouds will wrap around the local screen.  If not in start screen mode, the clouds wrap around once they exit the world.	
		if (self.bounds.x + self.bounds.width) > XRES + (self.bounds.width * 1.5):
			self.trueX = -1.5 * self.bounds.width
		if self.bounds.x < -1 * (self.bounds.width * 1.5):
			self.trueX = XRES
		self.trueY = 100 + ((self.energy / self.maxEnergy) * 50.0)
		self.bounds.y = self.trueY
			
	def setWind(self, magnitude, direction):
		"""
			Set the wind's magnitude and direction for this cloud.
		"""
		self.windStrength = magnitude
		self.windDirection = direction
		
	def flow(self, speed, angle):
		"""
			Move the cloud by the given speed and angle of the wind.
		"""
		radians = float(angle) / 57.295
		xDisp = math.cos(radians) * speed * self.mySpeed
		yDisp = math.sin(radians) * speed * self.mySpeed
		self.trueX += xDisp
		self.trueY -= yDisp
		self.bounds.x = int(self.trueX)
		self.bounds.y = int(self.trueY)
		
	def calculateDistance(self, point1, point2):
		"""
			Calculate the absolute value of the distance between point 1 and point 2.
		"""
		return math.sqrt(((point1[0] - point2[0]) ** 2) +  ((point1[1] - point2[1]) ** 2))
