#engine.Gameworld.py

import os, sys
import pygame

class Gameworld:
	"""
		Abstract class of a game screen.  A Gameworld should contain methods for returning sprites and handling game logic.  A Gameworld is linked to Driver by the self.controller variable.  The controller can be accessed if a scene needs to be changed or another scene needs to be modified.
	"""

	def __init__(self):
		"""
			Create a Gameworld without a controller for it being set yet.
		"""
		self.controller = -1
		
	def getSprites(self):
		"""
			Returns all sprites that need to be drawn.
		"""
		pass
		
	def setController(self, master):
		"""
			Sets the controller of this Gameworld to be "master."
		"""
		self.controller = master
		
	def tick(self, delta):
		"""
			Inform residents of the Gameworld that "delta" milliseconds have passed since the last call to tick(delta).
		"""
		pass
		
	def handleEvent(self, event):
		"""
			Inform residents of the Gameworld that "event" has occured.
		"""
		pass
		
	def draw(self, screen):
		"""
			Draws the images of this Gameworld to "screen."
		"""
		pass